#!/usr/bin/python
import importlib
import pathlib
import re
import sys

from .module_utils.module import AnsibleModule

sys.path.append(".")
mindocstring = "DOCUMENTATION = ''''''"
moduledir = pathlib.Path("plugins/modules")
regex = re.compile(
    "(?P<type>DOCUMENTATION|RETURNS) *= *r?(?P<quote>\"{3}|'{3})(---)?.*?(?P=quote)",
    re.MULTILINE | re.DOTALL,
)


def main() -> None:
    try:
        modules = list(moduledir.iterdir())
    except:
        print("failed to find modules")
        return
    for modfile in modules:
        if modfile.name in ("__init__.py", "__pycache__", "unit.py.example"):
            continue
        mod = importlib.import_module(".".join((modfile.parts[:-1]) + (modfile.stem,)))
        module: AnsibleModule
        if hasattr(mod, "Module"):
            module = mod.Module
        elif hasattr(mod, "__module_name__"):
            module = getattr(mod, mod.__module_name__)
        else:
            print("Error loading Module of File: {}. No Module or __module_name__ defined".format(modfile))
            continue
        try:
            moddoc = module.doc()
            returns = module.returns()
        except AttributeError:
            print("Broken module. skipping {}".format(modfile))
            continue
        except Exception as e:
            print("Error in documentation of module {}: {}".format(modfile, e))
            continue
        moddata = modfile.read_text()
        changed = False
        start = 0
        while True:
            match = regex.search(moddata, pos=start)
            if not match:
                break
            changed = True
            type = match.group("type")
            content = ""
            if type == "DOCUMENTATION":
                content = moddoc
            elif type == "RETURNS":
                content = returns
            else:
                Exception("Please update the script: Unknown Type")
            after = moddata[match.end() :]
            moddata = "{pre}{type} = r{quote}{doc}{quote}".format(
                pre=moddata[: match.start()],
                type=type,
                quote=match.group("quote"),
                doc=content,
            )
            start = len(moddata)
            moddata = f"{moddata}{after}"
        if changed is False:
            print(
                "no Documentation set for module {}. Please add at least \"{}\" to the file".format(
                    modfile.stem, mindocstring
                )
            )
            continue

        modfile.write_text(moddata)
        print("updated the documentation of module {}".format(module.name))
